"""Constants for feeds and feed entries."""

ATTR_ATTRIBUTION = "attribution"

CUSTOM_ATTRIBUTE = "custom_attribute"

DEFAULT_REQUEST_TIMEOUT = 10

UPDATE_OK = "OK"
UPDATE_OK_NO_DATA = "OK_NO_DATA"
UPDATE_ERROR = "ERROR"

XML_ATTR_PUBLICID = "@publicID"

XML_CDATA = "#text"

XML_TAG_Q_QUAKEML = "q:quakeml"
XML_TAG_AGENCYID = "agencyID"
XML_TAG_AUTHOR = "author"
XML_TAG_CREATIONINFO = "creationInfo"
XML_TAG_CREATIONTIME = "creationTime"
XML_TAG_DEPTH = "depth"
XML_TAG_DEPTHTYPE = "depthType"
XML_TAG_DESCRIPTION = "description"
XML_TAG_EVENT = "event"
XML_TAG_EVALUATIONMODE = "evaluationMode"
XML_TAG_EVALUATIONSTATUS = "evaluationStatus"
XML_TAG_EVENTPARAMETERS = "eventParameters"
XML_TAG_LATITUDE = "latitude"
XML_TAG_LONGITUDE = "longitude"
XML_TAG_MAG = "mag"
XML_TAG_MAGNITUDE = "magnitude"
XML_TAG_ORIGIN = "origin"
XML_TAG_STATIONCOUNT = "stationCount"
XML_TAG_TEXT = "text"
XML_TAG_TIME = "time"
XML_TAG_TYPE = "type"
XML_TAG_VALUE = "value"
